@@ -38,20 +38,14 @@ IO_THREAD = "IO_THREAD"
 SQL_THREAD = "SQL_THREAD"
 
 @_server.server_logging
-def get_master_status(server, options=None):
+def get_master_status(server):
     """Return the master status. In order to ease the navigation through
     the result set, a named tuple is always returned. Look up the `SHOW
     MASTER STATUS` command in the MySQL Manual for further details.
 
     :param server: MySQL Server.
-    :param options: Define how the result is formatted and retrieved.
-                    See :meth:`~mysql.fabric.server.MySQLServer.exec_stmt`.
     """
-    if options is None:
-        options = {}
-    options["columns"] = True
-    options["raw"] = False
-    return server.exec_stmt("SHOW MASTER STATUS", options)
+    return server.exec_stmt("SHOW MASTER STATUS", {"columns" : True})
 
 @_server.server_logging
 def reset_master(server):
@@ -70,23 +64,6 @@ def has_appropriate_privileges(server):
     """
     return server.has_privileges(["REPLICATION SLAVE"])
 
-@_server.server_logging
-def get_master_slaves(server, options=None):
-    """Return the slaves registered for this master. In order to ease the
-    navigation through the result set, a named tuple is always returned.
-    Please, look up the `SHOW SLAVE HOSTS` in the MySQL Manual for further
-    details.
-
-    :param server: MySQL Server.
-    :param options: Define how the result is formatted and retrieved.
-                    See :meth:`~mysql.fabric.server.MySQLServer.exec_stmt`.
-    """
-    if options is None:
-        options = {}
-    options["columns"] = True
-    options["raw"] = False
-    return server.exec_stmt("SHOW SLAVE HOSTS", options)
-
 @_server.server_logging
 def check_master_issues(server):
     """Check if there is any issue to make the server a master.
@@ -100,14 +77,15 @@ def check_master_issues(server):
 
     The dictionary returned may have the following keys::
 
-      status["is_running"] = False
-      status["is_binlog_enabled"] = False
-      status["is_gtid_enabled"] = False
-      status["is_slave_updates_enabled"] = False
-      status["rpl_user"] = False
+      status['is_not_running'] = False
+      status['is_binlog_not_enabled'] = False
+      status['is_gtid_not_enabled'] = False
+      status['is_slave_updates_not_enabled'] = False
+      status['no_rpl_user'] = False
 
     :param server: MySQL Server.
-    :return: A dictionary with issues, if there is any.
+    :return: Whether there is an issue or not and a dictionary with issues,
+             if there is any.
 
     .. note::
 
@@ -116,29 +94,36 @@ def check_master_issues(server):
        characteristics before considering a server eligible for becoming a
        master.
     """
-    status = {}
+    status = {
+        'is_not_running' : False,
+        'is_binlog_not_enabled' : False,
+        'is_gtid_not_enabled' : False,
+        'is_slave_updates_not_enabled' : False,
+        'no_rpl_user' : False
+    }
 
     if not server.is_connected():
-        status["is_running"] = False
-        return status
+        status["is_not_running"] = True
+        return True, status
 
     # Check for binlog.
     if not server.binlog_enabled:
-        status["is_binlog_enabled"] = False
+        status["is_binlog_not_enabled"] = True
 
     # Check for gtid.
     if not server.gtid_enabled:
-        status["is_gtid_enabled"] = False
+        status["is_gtid_not_enabled"] = True
 
     # Check for slave updates.
     if not server.get_variable("LOG_SLAVE_UPDATES"):
-        status["is_slave_updates_enabled"] = False
+        status["is_slave_updates_not_enabled"] = True
 
     # See if the current user has the appropriate replication privilege(s)
     if not has_appropriate_privileges(server):
-        status["rpl_user"] = False
+        status["no_rpl_user"] = True
 
-    return status
+    error = not all([v is False for v in status.itervalues()])
+    return error, status
 
 @_server.server_logging
 def get_slave_status(server, options=None):
@@ -150,11 +135,7 @@ def get_slave_status(server, options=None):
     :param options: Define how the result is formatted and retrieved.
                     See :meth:`~mysql.fabric.server.MySQLServer.exec_stmt`.
     """
-    if options is None:
-        options = {}
-    options["columns"] = True
-    options["raw"] = False
-    return server.exec_stmt("SHOW SLAVE STATUS", options)
+    return server.exec_stmt("SHOW SLAVE STATUS", {"columns" : True})
 
 @_server.server_logging
 def is_slave_thread_running(server, threads=None):
@@ -175,8 +156,9 @@ def slave_has_master(server):
     ret = get_slave_status(server)
     if ret:
         try:
-            _uuid.UUID(ret[0].Master_UUID)
-            return ret[0].Master_UUID
+            str_uuid = ret[0].Master_UUID
+            _uuid.UUID(str_uuid)
+            return str_uuid
         except ValueError:
             pass
     return None
@@ -353,18 +335,19 @@ def wait_for_slave(server, binlog_file, binlog_pos, timeout=0):
     """
     # Wait for slave to read the master log file
     res = server.exec_stmt(_MASTER_POS_WAIT,
-        {"params": (binlog_file, binlog_pos, timeout), "raw" : False })
+        {"params": (binlog_file, binlog_pos, timeout)}
+    )
 
     if res is None or res[0] is None or res[0][0] is None:
         raise _errors.DatabaseError(
             "Error waiting for slave to catch up. Binary log (%s, %s)." %
             (binlog_file, binlog_pos)
-            )
+        )
     elif res[0][0] == -1:
         raise _errors.TimeoutError(
             "Error waiting for slave to catch up. Binary log (%s, %s)." %
             (binlog_file, binlog_pos)
-            )
+        )
 
     assert(res[0][0] > -1)
 
@@ -440,19 +423,18 @@ def wait_for_slave_gtid(server, gtids, timeout=0):
             "Global Transaction IDs are not supported."
             )
 
-    res = server.exec_stmt(_GTID_WAIT,
-        {"params": (gtids, timeout), "raw" : False })
+    res = server.exec_stmt(_GTID_WAIT, {"params": (gtids, timeout)})
 
     if res is None or res[0] is None or res[0][0] is None:
         raise _errors.DatabaseError(
             "Error waiting for slave to catch up. "
             "GTID (%s)." % (gtids, )
-            )
+        )
     elif res[0][0] == -1:
         raise _errors.TimeoutError(
             "Error waiting for slave to catch up. "
             "GTID (%s)." % (gtids, )
-            )
+        )
 
     assert(res[0][0] > -1)
 
@@ -516,39 +498,47 @@ def check_slave_issues(server):
 
     The dictionary returned may have the following keys::
 
-      status["is_running"] = False
-      status["is_configured"] = False
-      status["io_running"] = False
-      status["sql_running"] = False
+      status["is_not_running"] = False
+      status["is_not_configured"] = False
+      status["io_not_running"] = False
+      status["sql_not_running"] = False
       status["io_error"] = False
       status["sql_error"] = False
 
     :param server: MySQL Server.
-    :return: A dictionary with issues, if there is any.
+    :return: Whether there is an issue or not and a dictionary with the
+             issues, if there is any.
     """
-    status = {}
+    status = {
+        'is_not_running': False,
+        'is_not_configured': False,
+        'io_not_running': False,
+        'sql_not_running': False,
+        'io_error': False,
+        'sql_error': False
+    }
 
     if not server.is_connected():
-        status["is_running"] = False
-        return status
+        status["is_not_running"] = True
+        return True, status
 
     ret = get_slave_status(server)
 
     if not ret:
-        status["is_configured"] = False
-        return status
+        status["is_not_configured"] = True
+        return True, status
 
-    # Check slave status for errors, threads activity
     if ret[0].Slave_IO_Running.upper() != "YES":
-        status["io_running"] = False
+        status["io_not_running"] = True
     if ret[0].Slave_SQL_Running.upper() != "YES":
-        status["sql_running"] = False
-    if ret[0].Last_IO_Errno > 0:
+        status["sql_not_running"] = True
+    if ret[0].Last_IO_Errno and ret[0].Last_IO_Errno > 0:
         status["io_error"] = ret[0].Last_IO_Error
-    if ret[0].Last_SQL_Errno > 0:
+    if ret[0].Last_SQL_Errno and ret[0].Last_SQL_Errno > 0:
         status["sql_error"] = ret[0].Last_SQL_Error
 
-    return status
+    error = not all([v is False for v in status.itervalues()])
+    return error, status
 
 @_server.server_logging
 def check_slave_delay(slave, master):
@@ -557,32 +547,36 @@ def check_slave_delay(slave, master):
     It checks if both the master and slave are alive and kicking, whether
     the `SQL_THREAD` and `IO_THREAD` are running or not. It reports the
     `SQL_Delay`, `Seconds_Behind_Master` and finally if GTIDs are enabled
-    the number of transactions behind master or the number of bytes behind
-    master otherwise.
+    the number of transactions behind master.
 
     The dictionary returned may have the following keys::
 
-      status["is_running"] = False
-      status["is_configured"] = False
+      status["is_not_running"] = False
+      status["is_not_configured"] = False
       status["sql_delay"] = Value
       status["seconds_behind"] = Value
-      status["bytes_behind"] = Value
       status["gtids_behind"] = Value
 
     :param slave: Reference to a slave (MySQL Server).
     :param master: Reference to the master (MySQL Server).
     :return: A dictionary with delays, if there is any.
     """
-    status = {}
+    status = {
+        'is_not_running': False,
+        'is_not_configured': False,
+        'sql_delay': 0,
+        'seconds_behind': 0,
+        'gtids_behind': 0
+    }
 
     if not slave.is_connected() or not master.is_connected():
-        status["is_running"] = False
+        status["is_not_running"] = True
         return status
 
     slave_status = get_slave_status(slave)
 
     if not slave_status:
-        status["is_configured"] = False
+        status["is_not_configured"] = True
         return status
 
     # Check if the slave must lag behind the master.
@@ -632,11 +626,13 @@ def _check_condition(server, threads, check_if_running):
     if ret:
         io_status = ret[0].Slave_IO_Running.upper() == check_stmt
         io_error = ret[0].Last_IO_Error
-        io_errno = int(ret[0].Last_IO_Errno)
+        io_errno = ret[0].Last_IO_Errno
+        io_errno = io_errno if io_errno else 0
 
         sql_status = ret[0].Slave_SQL_Running.upper() == check_stmt
         sql_error = ret[0].Last_SQL_Error
-        sql_errno = int(ret[0].Last_SQL_Errno)
+        sql_errno = ret[0].Last_SQL_Errno
+        sql_errno = sql_errno if sql_errno else 0
 
     achieved = True
     if SQL_THREAD in threads:
@@ -672,9 +668,9 @@ def _check_status_condition(server, thread, status):
     if SQL_THREAD == thread:
         sql_status = True if status in ret[0].Slave_SQL_Running_State else False
         sql_error = ret[0].Last_SQL_Error
-        sql_errno = int(ret[0].Last_SQL_Errno)
+        sql_errno = ret[0].Last_SQL_Errno
 
-        if sql_errno != 0:
+        if sql_errno and sql_errno != 0:
             raise _errors.DatabaseError(sql_error)
 
         achieved = sql_status
@@ -682,9 +678,9 @@ def _check_status_condition(server, thread, status):
     elif IO_THREAD == thread:
         io_status = True if status in ret[0].Slave_IO_State else False
         io_error = ret[0].Last_IO_Error
-        io_errno = int(ret[0].Last_IO_Errno)
+        io_errno = ret[0].Last_IO_Errno
 
-        if io_errno != 0:
+        if io_errno and io_errno != 0:
             raise _errors.DatabaseError(io_error)
 
         achieved = io_status
