@@ -21,6 +21,11 @@ prune operations.
 """
 
 import logging
+import time
+
+from mysql.connector.errorcode import (
+    ER_NO_SUCH_TABLE,
+)
 
 from mysql.fabric import (
     errors as _errors,
@@ -41,15 +46,12 @@ from mysql.fabric.sharding import (
     RangeShardingSpecification,
     HashShardingSpecification,
     Shards,
-    RangeShardingIntegerHandler,
-    HashShardingIntegerHandler,
     SHARDING_DATATYPE_HANDLER,
     SHARDING_SPECIFICATION_HANDLER,
 )
 
 from mysql.fabric.command import (
     ProcedureShard,
-    Command,
 )
 
 from mysql.fabric.services import (
@@ -77,15 +79,24 @@ class PruneShardTables(ProcedureShard):
         :param synchronous: Whether one should wait until the execution finishes
                             or not.
         """
+        prune_limit =   _services_utils.read_config_value(
+                                self.config,
+                                'sharding',
+                                'prune_limit'
+                            )
         procedures = _events.trigger(
-            PRUNE_SHARD_TABLES, self.get_lockable_objects(), table_name
+            PRUNE_SHARD_TABLES,
+            self.get_lockable_objects(),
+            table_name,
+            prune_limit
         )
         return self.wait_for_procedures(procedures, synchronous)
 
 CHECK_SHARD_INFORMATION = _events.Event("CHECK_SHARD_INFORMATION")
 BACKUP_SOURCE_SHARD = _events.Event("BACKUP_SOURCE_SHARD")
 RESTORE_SHARD_BACKUP = _events.Event("RESTORE_SHARD_BACKUP")
-SETUP_MOVE_SYNC = _events.Event("SETUP_MOVE_SYNC")
+SETUP_REPLICATION = _events.Event("SETUP_REPLICATION")
+SETUP_SYNC = _events.Event("SETUP_SYNC")
 SETUP_RESHARDING_SWITCH = _events.Event("SETUP_RESHARDING_SWITCH")
 PRUNE_SHARDS = _events.Event("PRUNE_SHARDS")
 class MoveShardServer(ProcedureShard):
@@ -125,7 +136,7 @@ class MoveShardServer(ProcedureShard):
         procedures = _events.trigger(
             CHECK_SHARD_INFORMATION, self.get_lockable_objects(), shard_id,
             group_id, mysqldump_binary, mysqlclient_binary, None, config_file,
-            "MOVE", update_only
+            "", "MOVE", update_only
         )
         return self.wait_for_procedures(procedures, synchronous)
 
@@ -141,7 +152,7 @@ class SplitShardServer(ProcedureShard):
     """
     group_name = "sharding"
     command_name = "split_shard"
-    def execute(self, shard_id,  group_id,  split_value = None,
+    def execute(self, shard_id, group_id, split_value=None,
                 update_only=False, synchronous=True):
         """Split the shard represented by the shard_id into the destination
         group.
@@ -159,22 +170,27 @@ class SplitShardServer(ProcedureShard):
                                 'sharding',
                                 'mysqldump_program'
                             )
-        mysqlclient_binary =  _services_utils.read_config_value(
+        mysqlclient_binary = _services_utils.read_config_value(
                                 self.config,
                                 'sharding',
                                 'mysqlclient_program'
                             )
+        prune_limit =   _services_utils.read_config_value(
+                                self.config,
+                                'sharding',
+                                'prune_limit'
+                            )
         config_file = self.config.config_file if self.config.config_file else ""
 
         procedures = _events.trigger(
             CHECK_SHARD_INFORMATION, self.get_lockable_objects(),
             shard_id, group_id, mysqldump_binary, mysqlclient_binary,
-            split_value, config_file, "SPLIT", update_only)
+            split_value, config_file, prune_limit, "SPLIT", update_only)
         return self.wait_for_procedures(procedures, synchronous)
 
 
 @_events.on_event(PRUNE_SHARD_TABLES)
-def _prune_shard_tables(table_name):
+def _prune_shard_tables(table_name, prune_limit):
     """Delete the data from the copied data directories based on the
     sharding configuration uploaded in the sharding tables of the state
     store. The basic logic consists of
@@ -185,14 +201,24 @@ def _prune_shard_tables(table_name):
         server.
 
     :param table_name: The table_name who's shards need to be pruned.
+    :param prune_limit: The number of DELETEs that should be
+                    done in one batch.
     """
     shard_mapping = ShardMapping.fetch(table_name)
-    SHARDING_SPECIFICATION_HANDLER[shard_mapping.type_name].\
-        delete_from_shard_db(table_name, shard_mapping.type_name)
+    try:
+        SHARDING_SPECIFICATION_HANDLER[shard_mapping.type_name].\
+            delete_from_shard_db(table_name, shard_mapping.type_name, prune_limit)
+    except _errors.DatabaseError as error:
+        if error.errno == ER_NO_SUCH_TABLE:
+            #Error happens because the actual tables are not present in the
+            #server. We will ignore this.
+            pass
+        else:
+            raise error
 
 @_events.on_event(CHECK_SHARD_INFORMATION)
-def _check_shard_information(shard_id,  destn_group_id, mysqldump_binary,
-                             mysqlclient_binary, split_value, config_file, cmd,
+def _check_shard_information(shard_id, destn_group_id, mysqldump_binary,
+                             mysqlclient_binary, split_value, config_file, prune_limit, cmd,
                              update_only):
     """Verify the sharding information before starting a re-sharding operation.
 
@@ -204,6 +230,8 @@ def _check_shard_information(shard_id,  destn_group_id, mysqldump_binary,
                         should be split.
     :param config_file: The complete path to the fabric configuration
                         file.
+    :param prune_limit: The number of DELETEs that should be
+                        done in one batch.
     :param cmd: Indicates if it is a split or a move being executed.
     :param update_only: If the operation is a update only operation.
     """
@@ -216,9 +244,9 @@ def _check_shard_information(shard_id,  destn_group_id, mysqldump_binary,
                 _services_sharding.MYSQLCLIENT_NOT_FOUND % mysqlclient_binary)
 
     if cmd == "SPLIT":
-        range_sharding_spec, _,  shard_mappings, _ = \
-            _services_sharding._verify_and_fetch_shard(shard_id)
-        upper_bound =\
+        range_sharding_spec, _, shard_mappings, _ = \
+            _services_sharding.verify_and_fetch_shard(shard_id)
+        upper_bound = \
             SHARDING_SPECIFICATION_HANDLER[shard_mappings[0].type_name].\
                         get_upper_bound(
                             range_sharding_spec.lower_bound,
@@ -243,14 +271,13 @@ def _check_shard_information(shard_id,  destn_group_id, mysqldump_binary,
                 upper_bound = HashShardingSpecification.fetch_max_key(shard_id)
 
             #Calculate the split value.
-            split_value =\
+            split_value = \
                 SHARDING_DATATYPE_HANDLER[shard_mappings[0].type_name].\
                 split_value(
                     range_sharding_spec.lower_bound,
                     upper_bound
                 )
-        elif shard_mappings[0].type_name == "RANGE" and\
-            split_value is not None:
+        elif split_value is not None:
             if not (SHARDING_DATATYPE_HANDLER[shard_mappings[0].type_name].\
                     is_valid_split_value(
                         split_value, range_sharding_spec.lower_bound,
@@ -261,13 +288,13 @@ def _check_shard_information(shard_id,  destn_group_id, mysqldump_binary,
                     _services_sharding.INVALID_LOWER_BOUND_VALUE %
                     (split_value, )
                 )
-        elif shard_mappings[0].type_name == "RANGE" and split_value is None:
+        elif split_value is None:
             raise _errors.ShardingError(
                 _services_sharding.SPLIT_VALUE_NOT_DEFINED
             )
 
     #Ensure that the group does not already contain a shard.
-    if (Shards.lookup_shard_id(destn_group_id) is not None):
+    if Shards.lookup_shard_id(destn_group_id) is not None:
         raise _errors.ShardingError(
             _services_sharding.SHARD_MOVE_DESTINATION_NOT_EMPTY %
             (destn_group_id, )
@@ -282,7 +309,6 @@ def _check_shard_information(shard_id,  destn_group_id, mysqldump_binary,
 
     #Fetch the group_id and the group that hosts the source shard.
     source_group_id = source_shard.group_id
-    source_group = Group.fetch(source_group_id)
 
     destn_group = Group.fetch(destn_group_id)
     if destn_group is None:
@@ -294,18 +320,18 @@ def _check_shard_information(shard_id,  destn_group_id, mysqldump_binary,
         _events.trigger_within_procedure(
             BACKUP_SOURCE_SHARD, shard_id, source_group_id, destn_group_id,
             mysqldump_binary, mysqlclient_binary, split_value, config_file,
-            cmd, update_only
+            prune_limit, cmd, update_only
         )
     else:
         _events.trigger_within_procedure(
             SETUP_RESHARDING_SWITCH, shard_id, source_group_id, destn_group_id,
-            split_value, cmd, update_only
+            split_value, prune_limit, cmd, update_only
         )
 
 @_events.on_event(BACKUP_SOURCE_SHARD)
 def _backup_source_shard(shard_id, source_group_id, destn_group_id,
                          mysqldump_binary, mysqlclient_binary, split_value,
-                         config_file, cmd, update_only):
+                         config_file, prune_limit, cmd, update_only):
     """Backup the source shard.
 
     :param shard_id: The shard ID of the shard that needs to be moved.
@@ -318,6 +344,8 @@ def _backup_source_shard(shard_id, source_group_id, destn_group_id,
                         particular shard will be split. Will be set only
                         for shard split operations.
     :param config_file: The complete path to the fabric configuration file.
+    :param prune_limit: The number of DELETEs that should be
+                        done in one batch.
     :param cmd: Indicates the type of re-sharding operation (move, split)
     :update_only: Only update the state store and skip provisioning.
     """
@@ -342,13 +370,14 @@ def _backup_source_shard(shard_id, source_group_id, destn_group_id,
                                      backup_image.path,
                                      split_value,
                                      config_file,
+                                     prune_limit,
                                      cmd
                                      )
 
 @_events.on_event(RESTORE_SHARD_BACKUP)
-def _restore_shard_backup(shard_id,  source_group_id, destn_group_id,
+def _restore_shard_backup(shard_id, source_group_id, destn_group_id,
                             mysqlclient_binary, backup_image,
-                            split_value, config_file, cmd):
+                            split_value, config_file, prune_limit, cmd):
     """Restore the backup on the destination Group.
 
     :param shard_id: The shard ID of the shard that needs to be moved.
@@ -362,6 +391,8 @@ def _restore_shard_backup(shard_id,  source_group_id, destn_group_id,
                         particular shard will be split. Will be set only
                         for shard split operations.
     :param config_file: The complete path to the fabric configuration file.
+    :param prune_limit: The number of DELETEs that should be
+                        done in one batch.
     :param cmd: Indicates the type of re-sharding operation
     """
     destn_group = Group.fetch(destn_group_id)
@@ -383,17 +414,18 @@ def _restore_shard_backup(shard_id,  source_group_id, destn_group_id,
 
     #Setup sync between the source and the destination groups.
     _events.trigger_within_procedure(
-                                     SETUP_MOVE_SYNC,
+                                     SETUP_REPLICATION,
                                      shard_id,
                                      source_group_id,
                                      destn_group_id,
                                      split_value,
+                                     prune_limit,
                                      cmd
                                      )
 
-@_events.on_event(SETUP_MOVE_SYNC)
-def _setup_move_sync(shard_id, source_group_id, destn_group_id, split_value,
-                                        cmd):
+@_events.on_event(SETUP_REPLICATION)
+def _setup_replication(shard_id, source_group_id, destn_group_id, split_value,
+                                        prune_limit, cmd):
     """Setup replication between the source and the destination groups and
     ensure that they are in sync.
 
@@ -404,6 +436,8 @@ def _setup_move_sync(shard_id, source_group_id, destn_group_id, split_value,
     :param split_value: Indicates the value at which the range for the
                         particular shard will be split. Will be set only
                         for shard split operations.
+    :param prune_limit: The number of DELETEs that should be
+                        done in one batch.
     :param cmd: Indicates the type of re-sharding operation
     """
     source_group = Group.fetch(source_group_id)
@@ -414,7 +448,7 @@ def _setup_move_sync(shard_id, source_group_id, destn_group_id, split_value,
     destination_group = Group.fetch(destn_group_id)
     if destination_group is None:
         raise _errors.ShardingError(_services_sharding.SHARD_GROUP_NOT_FOUND %
-                                    (destination_group_id, ))
+                                    (destn_group_id, ))
 
     master = MySQLServer.fetch(source_group.master)
     if master is None:
@@ -433,11 +467,61 @@ def _setup_move_sync(shard_id, source_group_id, destn_group_id, split_value,
     _replication.reset_slave(slave, clean=True)
 
     #Change the master to the shard group master.
-    _replication.switch_master(slave,  master,  master. user,  master.passwd)
+    _replication.switch_master(slave, master, master.user, master.passwd)
 
     #Start the slave so that syncing of the data begins
     _replication.start_slave(slave, wait=True)
 
+    #Setup sync between the source and the destination groups.
+    _events.trigger_within_procedure(
+                                     SETUP_SYNC,
+                                     shard_id,
+                                     source_group_id,
+                                     destn_group_id,
+                                     split_value,
+                                     prune_limit,
+                                     cmd
+                                     )
+
+@_events.on_event(SETUP_SYNC)
+def _setup_sync(shard_id, source_group_id, destn_group_id, split_value,
+                                        prune_limit, cmd):
+
+    """sync the source and the destination groups.
+
+    :param shard_id: The shard ID of the shard that needs to be moved.
+    :param source_group_id: The group_id of the source shard.
+    :param destn_group_id: The ID of the group to which the shard needs to
+                           be moved.
+    :param split_value: Indicates the value at which the range for the
+                        particular shard will be split. Will be set only
+                        for shard split operations.
+    :param prune_limit: The number of DELETEs that should be
+                        done in one batch.
+    :param cmd: Indicates the type of re-sharding operation
+    """
+    source_group = Group.fetch(source_group_id)
+    if source_group is None:
+        raise _errors.ShardingError(_services_sharding.SHARD_GROUP_NOT_FOUND %
+                                    (source_group_id, ))
+
+    destination_group = Group.fetch(destn_group_id)
+    if destination_group is None:
+        raise _errors.ShardingError(_services_sharding.SHARD_GROUP_NOT_FOUND %
+                                    (destn_group_id, ))
+
+    master = MySQLServer.fetch(source_group.master)
+    if master is None:
+        raise _errors.ShardingError(
+            _services_sharding.SHARD_GROUP_MASTER_NOT_FOUND)
+    master.connect()
+
+    slave = MySQLServer.fetch(destination_group.master)
+    if slave is None:
+        raise _errors.ShardingError(
+            _services_sharding.SHARD_GROUP_MASTER_NOT_FOUND)
+    slave.connect()
+
     #Synchronize until the slave catches up with the master.
     _replication.synchronize_with_read_only(slave, master)
 
@@ -452,12 +536,13 @@ def _setup_move_sync(shard_id, source_group_id, destn_group_id, split_value,
                                      source_group_id,
                                      destn_group_id,
                                      split_value,
+                                     prune_limit,
                                      cmd
                                      )
 
 @_events.on_event(SETUP_RESHARDING_SWITCH)
-def _setup_resharding_switch(shard_id,  source_group_id,  destination_group_id,
-                             split_value, cmd, update_only=False):
+def _setup_resharding_switch(shard_id, source_group_id, destination_group_id,
+                             split_value, prune_limit, cmd, update_only=False):
     """Setup the shard move or shard split workflow based on the command
     argument.
 
@@ -466,8 +551,8 @@ def _setup_resharding_switch(shard_id,  source_group_id,  destination_group_id,
     :param destination_group_id: The ID of the destination group.
     :param split_value: The value at which the shard needs to be split
                         (in the case of a shard split operation).
-    :param cmd: whether the operation that needs to be split is a
-                MOVE or a SPLIT operation.
+    :param prune_limit: The number of DELETEs that should be
+                        done in one batch.
     :param cmd: whether the operation that needs to be split is a
                 MOVE or a SPLIT operation.
     :update_only: Only update the state store and skip provisioning.
@@ -479,12 +564,12 @@ def _setup_resharding_switch(shard_id,  source_group_id,  destination_group_id,
         )
     elif cmd == "SPLIT":
         _setup_shard_switch_split(
-            shard_id,  source_group_id, destination_group_id, split_value,
-            cmd, update_only
+            shard_id, source_group_id, destination_group_id, split_value,
+            prune_limit, cmd, update_only
         )
 
-def _setup_shard_switch_split(shard_id,  source_group_id,  destination_group_id,
-                              split_value, cmd, update_only):
+def _setup_shard_switch_split(shard_id, source_group_id, destination_group_id,
+                              split_value, prune_limit, cmd, update_only):
     """Setup the moved shard to map to the new group.
 
     :param shard_id: The shard ID of the shard that needs to be moved.
@@ -494,12 +579,14 @@ def _setup_shard_switch_split(shard_id,  source_group_id,  destination_group_id,
     :param split_value: Indicates the value at which the range for the
                         particular shard will be split. Will be set only
                         for shard split operations.
+    :param prune_limit: The number of DELETEs that should be
+                        done in one batch.
     :param cmd: Indicates the type of re-sharding operation.
     :update_only: Only update the state store and skip provisioning.
     """
     #Fetch the Range sharding specification.
-    range_sharding_spec, source_shard,  shard_mappings,  shard_mapping_defn = \
-            _services_sharding._verify_and_fetch_shard(shard_id)
+    range_sharding_spec, source_shard, shard_mappings, shard_mapping_defn = \
+            _services_sharding.verify_and_fetch_shard(shard_id)
 
     #Disable the old shard
     source_shard.disable()
@@ -508,6 +595,20 @@ def _setup_shard_switch_split(shard_id,  source_group_id,  destination_group_id,
     range_sharding_spec.remove()
     source_shard.remove()
 
+    destination_group = Group.fetch(destination_group_id)
+    if destination_group is None:
+        raise _errors.ShardingError(_services_sharding.SHARD_GROUP_NOT_FOUND %
+                                    (destination_group_id, ))
+    destn_group_master = MySQLServer.fetch(destination_group.master)
+    if destn_group_master is None:
+        raise _errors.ShardingError(
+            _services_sharding.SHARD_GROUP_MASTER_NOT_FOUND)
+    destn_group_master.connect()
+
+    #Make the destination group as read only to disable updates until the
+    #connectors update their caches, thus avoiding inconsistency.
+    destn_group_master.read_only = True
+
     #Add the new shards. Generate new shard IDs for the shard being
     #split and also for the shard that is created as a result of the split.
     new_shard_1 = Shards.add(source_shard.group_id, "DISABLED")
@@ -546,6 +647,10 @@ def _setup_shard_switch_split(shard_id,  source_group_id,  destination_group_id,
             new_shard_2.shard_id
         )
 
+    #The sleep ensures that the connector have refreshed their caches with the
+    #new shards that have been added as a result of the split.
+    time.sleep(_utils.TTL)
+
     #The source shard group master would have been marked as read only
     #during the sync. Remove the read_only flag.
     source_group = Group.fetch(source_group_id)
@@ -558,11 +663,19 @@ def _setup_shard_switch_split(shard_id,  source_group_id,  destination_group_id,
         raise _errors.ShardingError(
             _services_sharding.SHARD_GROUP_MASTER_NOT_FOUND)
     source_group_master.connect()
+
+    #Kill all the existing connections on the servers
+    source_group.kill_connections_on_servers()
+
+    #Allow connections on the source group master
     source_group_master.read_only = False
 
+    #Allow connections on the destination group master
+    destn_group_master.read_only = False
+
     #Setup replication for the new group from the global server
     _group_replication.setup_group_replication \
-            (shard_mapping_defn[2],  destination_group_id)
+            (shard_mapping_defn[2], destination_group_id)
 
     #Enable the split shards
     new_shard_1.enable()
@@ -571,31 +684,50 @@ def _setup_shard_switch_split(shard_id,  source_group_id,  destination_group_id,
     #Trigger changing the mappings for the shard that was copied
     if not update_only:
         _events.trigger_within_procedure(
-            PRUNE_SHARDS, new_shard_1.shard_id, new_shard_2.shard_id
+            PRUNE_SHARDS, new_shard_1.shard_id, new_shard_2.shard_id, prune_limit
         )
 
 @_events.on_event(PRUNE_SHARDS)
-def _prune_shard_tables_after_split(shard_id_1, shard_id_2):
+def _prune_shard_tables_after_split(shard_id_1, shard_id_2, prune_limit):
     """Prune the two shards generated after a split.
 
     :param shard_id_1: The first shard id after the split.
     :param shard_id_2: The second shard id after the split.
+    :param prune_limit: The number of DELETEs that should be
+                        done in one batch.
     """
     #Fetch the Range sharding specification. When we start implementing
     #heterogenous sharding schemes, we need to find out the type of
     #sharding scheme and we should use that to find out the sharding
     #implementation.
-    _, _,  shard_mappings,  _ = _services_sharding.\
-        _verify_and_fetch_shard(shard_id_1)
+    _, _, shard_mappings,  _ = _services_sharding.\
+        verify_and_fetch_shard(shard_id_1)
 
     #All the shard mappings associated with this shard_id should be
     #of the same type. Hence it is safe to use one of them.
-    SHARDING_SPECIFICATION_HANDLER[shard_mappings[0].type_name].\
-    prune_shard_id(shard_id_1, shard_mappings[0].type_name)
-    SHARDING_SPECIFICATION_HANDLER[shard_mappings[0].type_name].\
-    prune_shard_id(shard_id_2, shard_mappings[0].type_name)
+    try:
+        SHARDING_SPECIFICATION_HANDLER[shard_mappings[0].type_name].\
+        prune_shard_id(shard_id_1, shard_mappings[0].type_name, prune_limit)
+    except _errors.DatabaseError as error:
+        if error.errno == ER_NO_SUCH_TABLE:
+            #Error happens because the actual tables are not present in the
+            #server. We will ignore this.
+            pass
+        else:
+            raise error
 
-def _setup_shard_switch_move(shard_id,  source_group_id, destination_group_id,
+    try:
+        SHARDING_SPECIFICATION_HANDLER[shard_mappings[0].type_name].\
+        prune_shard_id(shard_id_2, shard_mappings[0].type_name, prune_limit)
+    except _errors.DatabaseError as error:
+        if error.errno == ER_NO_SUCH_TABLE:
+            #Error happens because the actual tables are not present in the
+            #server. We will ignore this.
+            pass
+        else:
+            raise error
+
+def _setup_shard_switch_move(shard_id, source_group_id, destination_group_id,
                              update_only):
     """Setup the moved shard to map to the new group.
 
@@ -609,18 +741,34 @@ def _setup_shard_switch_move(shard_id,  source_group_id, destination_group_id,
     #heterogenous sharding schemes, we need to find out the type of
     #sharding scheme and we should use that to find out the sharding
     #implementation.
-    _, source_shard,  _,  shard_mapping_defn = \
-        _services_sharding._verify_and_fetch_shard(shard_id)
+    _, source_shard, _, shard_mapping_defn = \
+        _services_sharding.verify_and_fetch_shard(shard_id)
+
+    destination_group = Group.fetch(destination_group_id)
+    if destination_group is None:
+        raise _errors.ShardingError(_services_sharding.SHARD_GROUP_NOT_FOUND %
+                                    (destination_group_id, ))
+    destn_group_master = MySQLServer.fetch(destination_group.master)
+    if destn_group_master is None:
+        raise _errors.ShardingError(
+            _services_sharding.SHARD_GROUP_MASTER_NOT_FOUND)
+    destn_group_master.connect()
+    #Set the destination group master to read_only
+    destn_group_master.read_only = True
 
     #Setup replication between the shard group and the global group.
     _group_replication.setup_group_replication \
-            (shard_mapping_defn[2],  destination_group_id)
+            (shard_mapping_defn[2], destination_group_id)
     #set the shard to point to the new group.
     source_shard.group_id = destination_group_id
     #Stop the replication between the global server and the original
     #group associated with the shard.
     _group_replication.stop_group_slave\
-            (shard_mapping_defn[2],  source_group_id,  True)
+            (shard_mapping_defn[2], source_group_id,  True)
+
+    #The sleep ensures that the connector have refreshed their caches with the
+    #new shards that have been added as a result of the split.
+    time.sleep(_utils.TTL)
 
     #Reset the read only flag on the source server.
     source_group = Group.fetch(source_group_id)
@@ -636,3 +784,7 @@ def _setup_shard_switch_move(shard_id,  source_group_id, destination_group_id,
     if not update_only:
         master.connect()
         master.read_only = False
+        #Kill all the existing connections on the servers
+        source_group.kill_connections_on_servers()
+        #allow updates in the destination group master
+        destn_group_master.read_only = False
