@@ -624,6 +624,11 @@ def _undo_destroy_group(group_id):
 def _lookup_uuid(address, timeout):
     """Return server's uuid.
     """
+    try:
+        timeout = float(timeout)
+    except (TypeError, ValueError):
+        pass
+
     timeout = timeout or DEFAULT_UNREACHABLE_TIMEOUT
     try:
         return _server.MySQLServer.discover_uuid(
@@ -674,7 +679,7 @@ def _remove_server(group_id, server_id):
 
     _server.MySQLServer.remove(server)
     server.disconnect()
-    _server.ConnectionPool().purge_connections(server.uuid)
+    _server.ConnectionManager().purge_connections(server)
 
 @_events.on_event(SET_SERVER_STATUS)
 def _set_server_status(server_id, status, update_only):
