@@ -16,7 +16,7 @@
 #
 
 """
-This file contains the replication administration tools for managine a
+This file contains the replication administration commands to manage a
 simple master-to-slaves topology.
 """
 
@@ -29,7 +29,10 @@ from datetime import datetime, timedelta
 
 from mysql.utilities.exception import UtilRplError
 from mysql.utilities.common.ip_parser import hostname_is_ip
-from mysql.utilities.common.messages import ERROR_SAME_MASTER, HOST_IP_WARNING
+from mysql.utilities.common.messages import (ERROR_SAME_MASTER,
+                                             HOST_IP_WARNING,
+                                             EXTERNAL_SCRIPT_DOES_NOT_EXIST,
+                                             INSUFFICIENT_FILE_PERMISSIONS)
 from mysql.utilities.common.tools import ping_host, execute_script
 from mysql.utilities.common.format import print_list
 from mysql.utilities.common.topology import Topology
@@ -78,7 +81,7 @@ _DATE_LEN = 22
 
 _ERRANT_TNX_ERROR = "Errant transaction(s) found on slave(s)."
 
-_GTID_ON_REQ = "Slave election requires GTID_MODE=ON for all servers."
+_GTID_ON_REQ = "{action} requires GTID_MODE=ON for all servers."
 
 WARNING_SLEEP_TIME = 10
 
@@ -405,8 +408,9 @@ class RplCommands(object):
         is issued.
         """
         if not self.topology.gtid_enabled():
-            print("# WARNING: {0}".format(_GTID_ON_REQ))
-            self._report(_GTID_ON_REQ, logging.WARN, False)
+            warn_msg = _GTID_ON_REQ.format(action='Slave election')
+            print("# WARNING: {0}".format(warn_msg))
+            self._report(warn_msg, logging.WARN, False)
             return
 
         # Check for mixing IP and hostnames
@@ -446,15 +450,16 @@ class RplCommands(object):
             options = {}
         srv_list = self.topology.get_servers_with_gtid_not_on()
         if srv_list:
-            print("# ERROR: {0}".format(_GTID_ON_REQ))
-            self._report(_GTID_ON_REQ, logging.ERROR, False)
+            err_msg = _GTID_ON_REQ.format(action='Slave election')
+            print("# ERROR: {0}".format(err_msg))
+            self._report(err_msg, logging.ERROR, False)
             for srv in srv_list:
                 msg = "#  - GTID_MODE={0} on {1}:{2}".format(srv[2], srv[0],
                                                              srv[1])
                 self._report(msg, logging.ERROR)
 
-            self._report(_GTID_ON_REQ, logging.CRITICAL, False)
-            raise UtilRplError(_GTID_ON_REQ)
+            self._report(err_msg, logging.CRITICAL, False)
+            raise UtilRplError(err_msg)
 
         # Check for --master-info-repository=TABLE if rpl_user is None
         if not self._check_master_info_type():
@@ -732,14 +737,6 @@ class RplCommands(object):
                 WARNING_SLEEP_TIME))
             time.sleep(WARNING_SLEEP_TIME)
 
-        # Test failover script. If it doesn't exist, fail.
-        no_exec_fail_msg = "Failover check script cannot be found. Please " + \
-                           "check the path and filename for accuracy and " + \
-                           "restart the failover console."
-        if exec_fail is not None and not os.path.exists(exec_fail):
-            self._report(no_exec_fail_msg, logging.CRITICAL, False)
-            raise UtilRplError(no_exec_fail_msg)
-
         # Check existence of errant transactions on slaves
         errant_tnx = self.topology.find_errant_transactions()
         if errant_tnx:
@@ -777,9 +774,16 @@ class RplCommands(object):
             # using connectivity checks.
             if exec_fail is not None:
                 # Execute failover check script
-                if not os.path.exists(exec_fail):
-                    self._report(no_exec_fail_msg, logging.CRITICAL, False)
-                    raise UtilRplError(no_exec_fail_msg)
+                if not os.path.isfile(exec_fail):
+                    message = EXTERNAL_SCRIPT_DOES_NOT_EXIST.format(
+                        path=exec_fail)
+                    self._report(message, logging.CRITICAL, False)
+                    raise UtilRplError(message)
+                elif not os.access(exec_fail, os.X_OK):
+                    message = INSUFFICIENT_FILE_PERMISSIONS.format(
+                        path=exec_fail, permissions='execute')
+                    self._report(message, logging.CRITICAL, False)
+                    raise UtilRplError(message)
                 else:
                     self._report("# Spawning external script for failover "
                                  "checking.")
