@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,12 +19,13 @@
 This module contains and abstraction of a MySQL user object.
 """
 
-from collections import namedtuple, defaultdict
-
 import re
 
+from collections import namedtuple, defaultdict
+
 from mysql.utilities.exception import UtilError, UtilDBError, FormatError
 from mysql.utilities.common.ip_parser import parse_connection, clean_IPv6
+from mysql.utilities.common.messages import ERROR_USER_WITHOUT_PRIVILEGES
 from mysql.utilities.common.sql_transform import (is_quoted_with_backticks,
                                                   quote_with_backticks)
 
@@ -87,7 +88,8 @@ def parse_user_host(user_name):
 
 def grant_proxy_ssl_privileges(server, user, passw, at='localhost',
                                privs="ALL PRIVILEGES", grant_opt=True,
-                               ssl=True, grant_proxy=True):
+                               ssl=True, grant_proxy=True, proxy_user='root',
+                               proxy_host='localhost'):
     """Grant privileges to an user in a server with GRANT OPTION or/and
     REQUIRE SSL if required.
 
@@ -99,6 +101,8 @@ def grant_proxy_ssl_privileges(server, user, passw, at='localhost',
     grant_opt[in]      if True, it will grant with GRANT OPTION (default True).
     ssl[in]            if True, it will set REQUIRE SSL (default True).
     grant_proxy[in]    if True, it will grant GRANT PROXY (default True).
+    proxy_user[in]     username for the proxied account (default: root)
+    proxy_host[in]     hostname for the proxied account (default: localhost)
 
     Note: Raises UtilError on any Error.
     """
@@ -116,21 +120,66 @@ def grant_proxy_ssl_privileges(server, user, passw, at='localhost',
         server.exec_query(" ".join(grant))
     except UtilDBError as err:
         raise UtilError("Cannot create new user {0} at {1}:{2} reason:"
-                        "{3}".format(user, server.host, server.host,
+                        "{3}".format(user, server.host, server.port,
                                      err.errmsg))
 
     if grant_proxy:
-        grant = [
-            "GRANT PROXY ON ''@''",
-            "TO '{0}'@'{1}'".format(user, at),
-            "WITH GRANT OPTION"
-        ]
+        grant = ("GRANT PROXY ON '{0}'@'{1}' "
+                 "TO '{2}'@'{3}' "
+                 "WITH GRANT OPTION").format(proxy_user, proxy_host, user, at)
         try:
-            server.exec_query(" ".join(grant))
+            server.exec_query(grant)
         except UtilDBError as err:
             raise UtilError("Cannot grant proxy to user {0} at {1}:{2} "
                             "reason:{3}".format(user, server.host,
-                                                server.host, err.errmsg))
+                                                server.port, err.errmsg))
+
+
+def check_privileges(server, operation, privileges, description,
+                     verbosity=0, reporter=None):
+    """Check required privileges.
+
+    This method check if the used user possess the required privileges to
+    execute a statement or operation.
+    An exception is thrown if the user doesn't have enough privileges.
+
+    server[in]        Server instance to check.
+    operation[in]     The name of tha task that requires the privileges,
+                      used in the error message if an exception is thrown.
+    privileges[in]    List of the required privileges.
+    description[in]   Description of the operation requiring the User's
+                      privileges, used in the message if verbosity if given.
+    verbosity[in]     Verbosity.
+    reporter[in]      A method to invoke with messages and warnings
+                      (by default print).
+    """
+    # print message with the given reporter.
+    if reporter is None and verbosity > 0:
+        print("# Checking user permission to {0}...\n"
+              "#".format(description))
+    elif reporter is not None and verbosity > 0:
+        reporter("# Checking user permission to {0}...\n"
+                 "#".format(description))
+
+    # Check privileges
+    user_obj = User(server, "{0}@{1}".format(server.user, server.host))
+    need_privileges = []
+    for privilege in privileges:
+        if not user_obj.has_privilege('*', '*', privilege):
+            need_privileges.append(privilege)
+
+    if len(need_privileges) > 0:
+        if len(need_privileges) > 1:
+            privileges_needed = "{0} and {1}".format(
+                ", ".join(need_privileges[:-1]),
+                need_privileges[-1]
+            )
+        else:
+            privileges_needed = need_privileges[0]
+        raise UtilError(ERROR_USER_WITHOUT_PRIVILEGES.format(
+            user=server.user, host=server.host, port=server.port,
+            operation=operation, req_privileges=privileges_needed
+        ))
 
 
 class User(object):
@@ -239,7 +288,7 @@ class User(object):
         return (res is not None and len(res) >= 1)
 
     @staticmethod
-    def _get_grants_as_dict(grant_list):
+    def _get_grants_as_dict(grant_list, verbosity=0):
         """Transforms list of grant string statements into a dictionary.
 
         grant_list[in]    List of grant strings as returned from the server
@@ -249,8 +298,12 @@ class User(object):
         grant_dict = defaultdict(lambda: defaultdict(set))
         for grant in grant_list:
             grant_tpl = User._parse_grant_statement(grant[0])
-            # Ignore PROXY privilege
-            if 'PROXY' not in grant_tpl.privileges:
+            # Ignore PROXY privilege, it is not yet supported
+            if verbosity > 0:
+                if 'PROXY' in grant_tpl:
+                    print("#WARNING: PROXY privilege will be ignored.")
+            grant_tpl.privileges.discard('PROXY')
+            if grant_tpl.privileges:
                 grant_dict[grant_tpl.db][grant_tpl.object].update(
                     grant_tpl.privileges)
         return grant_dict
@@ -292,7 +345,8 @@ class User(object):
 
             # Cache user grants
             self.grant_list = grants[:]
-            self.grant_dict = User._get_grants_as_dict(self.grant_list)
+            self.grant_dict = User._get_grants_as_dict(self.grant_list,
+                                                       self.verbosity)
             # If current user is already using global host wildcard '%', there
             # is no need to run the show grants again.
             if globals_privs:
@@ -305,7 +359,7 @@ class User(object):
                             grants.append(grant)
                         self.global_grant_list = grants[:]
                         self.global_grant_dict = User._get_grants_as_dict(
-                            self.global_grant_list)
+                            self.global_grant_list, self.verbosity)
                     except UtilDBError:
                         # User has no global privs, return the just the ones
                         # for current host
@@ -548,7 +602,7 @@ class User(object):
             (?:(?:(\*|`?[^']+`?)\.(\*|`?[^']+`?)) # object where grant applies
             | ('[^']*'@'[^']*')) # For proxy grants user/host
             \sTO\s([^@]+@[\S]+) # grantee
-            (?:\sIDENTIFIED\sBY\sPASSWORD\s\'[^\']+\')? # optional password
+            (?:\sIDENTIFIED\sBY\sPASSWORD(?:\s\'[^\']+\')?)? # optional pwd
             (?:\sREQUIRE\sSSL)? # optional SSL
             (\sWITH\sGRANT\sOPTION)? # optional grant option
             $ # End of grant statement
@@ -556,7 +610,6 @@ class User(object):
 
         grant_tpl_factory = namedtuple("grant_info", "privileges proxy_user "
                                                      "db object user")
-        grants = None
         match = re.match(grant_parse_re, statement)
 
         if match:
